package cz.jzitnik.chronos.interactions.list;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import cz.jzitnik.chronos.entities.Character;
import cz.jzitnik.chronos.entities.Item;
import cz.jzitnik.chronos.entities.ItemType;
import cz.jzitnik.chronos.entities.Player;
import cz.jzitnik.chronos.interactions.InteractionPlayer;
import cz.jzitnik.chronos.payload.responses.InteractionResponse;
import cz.jzitnik.chronos.repository.CharacterRepository;
import cz.jzitnik.chronos.services.ItemService;
import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Random;
import java.util.Set;

@Service
public class Hangman implements InteractionPlayer {
    @Autowired
    private CharacterRepository characterRepository;
    @Autowired
    private ItemService itemService;

    private static final int ATTEMPTS = 6; // Arbitrary max incorrect guesses

    // This can be expanded at any time
    private final String[] words = {
        "dům", "pes", "les", "hra", "vlk", "kůň", "vůz", "most", "pán", "noc", "den", "čas", "hora",
        "sýr", "chleba", "pole", "věk", "slon", "osel", "šnek", "pták", "koza", "míč", "lípa", "knedlík",
        "čaj", "víno", "hráč", "strom", "nebe", "město", "stůl", "kniha", "kráva", "cesta", "divadlo",
        "okno", "hory", "slunce", "přítel", "rodina", "jahoda", "židle", "mravenec", "mléko", "sukně",
        "koberec", "tabule", "škola", "letadlo", "jablko", "písmeno", "košile", "ulice", "dopis",
        "zpráva", "kolega", "kamarád", "postel", "zmrzlina", "autobus", "tramvaj", "pohádka", "vánočka",
        "učitelka", "kancelář", "čokoláda", "letadlo", "zahrada", "studentka", "knihtisk", "televizor",
        "polévka", "zázračný", "papírnictví", "šampaňské", "kinematografie", "obloha", "křišťálový"
    };

    // Memory
    @AllArgsConstructor
    @NoArgsConstructor
    @Getter
    @Setter
    private static class HangmanMemory {
        private String word;
        private Set<java.lang.Character> correctGuessesSet;
        private Set<java.lang.Character> incorrectGuessesSet;
        private String currentProgress;
    }
    private static HangmanMemory readMemory(String memory) {
        ObjectMapper objectMapper = new ObjectMapper();
        try {
            return objectMapper.readValue(memory, HangmanMemory.class);
        } catch (Exception e) {
            throw new RuntimeException("Somebody just fucked up db."); // If this happens idk what is wrong with my life
        }
    }
    private static String writeMemory(HangmanMemory memory) {
        ObjectMapper objectMapper = new ObjectMapper();
        try {
            return objectMapper.writeValueAsString(memory);
        } catch (JsonProcessingException e) {
            throw new RuntimeException(e); // Well hopefully this will never happen
        }
    }

    // Response
    @AllArgsConstructor
    @Getter
    @Setter
    private static class HangmanResponse {
        public enum Type {
            GAME_CREATED,
            GAME_WON,
            GAME_LOST,
            GAME_CONTINUE,
        }

        private Type type;

        private Set<java.lang.Character> correctGuessesSet;
        private Set<java.lang.Character> incorrectGuessesSet;
        private String currentProgress;
    }

    @Override
    public InteractionResponse play(Player player, Character character, String data) {
        if (character.getInteractionData().getPlayer() == null) {
            // New game
            try {
                return initGame(player, character);
            } catch (JsonProcessingException e) {
                throw new RuntimeException(e);
            }
        }

        // Game already exists
        if (!character.getInteractionData().getPlayer().getId().equals(player.getId())) {
            // Another player tries to interact with character while somebody else is interacting with him
            return new InteractionResponse(false, "already_playing", new ArrayList<>());
        }

        try {
            return guessChar(player, character, data);
        } catch (JsonProcessingException e) {
            throw new RuntimeException(e); // I hate error handling. This code is just shit but whatever
        }
    }

    private InteractionResponse initGame(Player player, Character character) throws JsonProcessingException {
        var interactionData = character.getInteractionData();
        interactionData.setPlayer(player);

        Random random = new Random();
        int index = random.nextInt(words.length);
        String randomWord = words[index];

        String initialProgress = randomWord.replaceAll(".", "_");

        interactionData.setMemory(writeMemory(
                new HangmanMemory(randomWord, new HashSet<>(), new HashSet<>(), initialProgress)
        ));

        characterRepository.save(character);

        var response = new HangmanResponse(HangmanResponse.Type.GAME_CREATED, new HashSet<>(), new HashSet<>(), initialProgress);
        ObjectMapper objectMapper = new ObjectMapper();

        return new InteractionResponse(true, objectMapper.writeValueAsString(response), new ArrayList<>());
    }

    private InteractionResponse guessChar(Player player, Character character, String data) throws JsonProcessingException {
        var interactionMemory = readMemory(character.getInteractionData().getMemory());

        if (data.equals("get_data")) {
            var response = new HangmanResponse(HangmanResponse.Type.GAME_CONTINUE, interactionMemory.getCorrectGuessesSet(), interactionMemory.getIncorrectGuessesSet(), interactionMemory.getCurrentProgress());
            ObjectMapper objectMapper = new ObjectMapper();
            var responseText = objectMapper.writeValueAsString(response);

            return new InteractionResponse(false, responseText, new ArrayList<>());
        }

        String word = interactionMemory.getWord();
        var correctGuessesSet = interactionMemory.getCorrectGuessesSet();
        var incorrectGuessesSet = interactionMemory.getIncorrectGuessesSet();
        StringBuilder currentProgress = new StringBuilder(interactionMemory.getCurrentProgress());

        if (data.isEmpty()) {
            return new InteractionResponse(false, "invalid_input", new ArrayList<>());
        }

        char guessedChar = data.charAt(0);

        if (correctGuessesSet.contains(guessedChar) || incorrectGuessesSet.contains(guessedChar)) {
            return new InteractionResponse(false, "already_guessed", new ArrayList<>());
        }

        boolean isCorrect = false;
        for (int i = 0; i < word.length(); i++) {
            if (word.charAt(i) == guessedChar) {
                currentProgress.setCharAt(i, guessedChar);
                isCorrect = true;
            }
        }

        if (isCorrect) {
            correctGuessesSet.add(guessedChar);
        } else {
            incorrectGuessesSet.add(guessedChar);
        }

        interactionMemory.setCorrectGuessesSet(correctGuessesSet);
        interactionMemory.setIncorrectGuessesSet(incorrectGuessesSet);
        interactionMemory.setCurrentProgress(currentProgress.toString());

        character.getInteractionData().setMemory(writeMemory(interactionMemory));

        characterRepository.save(character);


        // Submit response
        ObjectMapper objectMapper = new ObjectMapper();

        if (currentProgress.toString().equals(word)) {
            var item = new Item(ItemType.KEY_FRAGMENT, player);
            itemService.addItem(player, item);

            character.setInteractedWith(true);
            characterRepository.save(character);

            var items = new ArrayList<Item>();
            items.add(item);

            var response = new HangmanResponse(HangmanResponse.Type.GAME_WON, correctGuessesSet, incorrectGuessesSet, currentProgress.toString());
            var responseText = objectMapper.writeValueAsString(response);
            return new InteractionResponse(true, responseText, items);
        } else if (incorrectGuessesSet.size() >= ATTEMPTS) {
            character.setInteractedWith(true);
            characterRepository.save(character);

            var response = new HangmanResponse(HangmanResponse.Type.GAME_LOST, correctGuessesSet, incorrectGuessesSet, currentProgress.toString());
            var responseText = objectMapper.writeValueAsString(response);
            return new InteractionResponse(false, responseText, new ArrayList<>());
        }

        var response = new HangmanResponse(HangmanResponse.Type.GAME_CONTINUE, correctGuessesSet, incorrectGuessesSet, currentProgress.toString());
        var responseText = objectMapper.writeValueAsString(response);

        return new InteractionResponse(false, responseText, new ArrayList<>());
    }
}
