package cz.jzitnik.chronos.services;

import cz.jzitnik.chronos.entities.*;
import cz.jzitnik.chronos.entities.Character;
import cz.jzitnik.chronos.interactions.Interaction;
import org.springframework.stereotype.Service;

import java.util.ArrayList;

@Service
public class InitGameService {
    public Room initGame(Game game) {
        var rooms = new ArrayList<Room>();

        // Outside
        var outside = new Room(
                "Outside", // Náměstí
                game
        );
        var outside_characters = new ArrayList<Character>();
        var elf_items = new ArrayList<Item>();
        var elf = new Character(
                "Skřítek",
                outside,
                "Ahoj já jsem skřítek... Můj dialog doplnit"
        );
        elf_items.add(new Item(ItemType.LUCK_POTION, elf));
        elf.setInventory(elf_items);
        outside_characters.add(elf);
        outside.setCharacters(outside_characters);

        // Stodola
        var stodola = new Room(
                "Stodola",
                game
        );
        var stodola_characters = new ArrayList<Character>();
        var farmar = new Character(
                "Farmář",
                stodola,
                "Ahoj já jsem farmář a budeš se mnou hrát kámen nůžky papír. Dialog doplnit."
        );
        farmar.setInteraction(Interaction.Farmer);
        farmar.setInteractionData(new cz.jzitnik.chronos.entities.Interaction(
                "Tak si zahrajeme kámen nůžky papír",
                "Se mnou někdo již hrál kámen nůžky papír. Další fragment klíče nemám.",
                farmar
        ));
        stodola_characters.add(farmar);
        stodola.setCharacters(stodola_characters);

        // Shop
        var shop = new Room(
                "Obchod",
                game
        );
        var shop_characters = new ArrayList<Character>();
        var cashier = new Character(
                "Prodavač",
                shop,
                "Ahoj já jsem prodavač a budeš se mnou hrát šibenici."
        );
        cashier.setInteraction(Interaction.Cashier);
        cashier.setInteractionData(new cz.jzitnik.chronos.entities.Interaction(
                "Tak si zahrajeme šibenici",
                "Se mnou někdo již hrál šibenici. Další fragment klíče nemám.",
                cashier
        ));
        shop_characters.add(cashier);
        shop.setCharacters(shop_characters);

        // Knihovna
        var library = new Room(
                "Knihovna",
                game
        );
        var library_characters = new ArrayList<Character>();
        var librarian = new Character(
                "Knihovník",
                library,
                "Ahoj já jsem knihovník a budeš se mnou hrát wordle."
        );
        librarian.setInteraction(Interaction.Librarian);
        librarian.setInteractionData(new cz.jzitnik.chronos.entities.Interaction(
                "Tak si zahrajeme wordle.",
                "Se mnou někdo již hrál wordle. Další fragment klíče nemám.",
                librarian
        ));
        library_characters.add(librarian);
        library.setCharacters(library_characters);

        // Hospoda
        var inn = new Room(
                "Hospoda",
                game
        );
        var inn_characters = new ArrayList<Character>();
        var innkeeper = new Character(
                "Hostinský",
                inn,
                "Ahoj já jsem hostinský a budeš se mnou hrát piškvorky."
        );
        innkeeper.setInteraction(Interaction.Innkeeper);
        innkeeper.setInteractionData(new cz.jzitnik.chronos.entities.Interaction(
                "Tak si zahrajeme piškvorky.",
                "Se mnou někdo již hrál piškvorky. Další fragment klíče nemám.",
                innkeeper
        ));
        inn_characters.add(innkeeper);
        inn.setCharacters(inn_characters);

        // Dům starosty
        var mayor_house = new Room(
                "Dům starosty",
                game
        );
        var mayor_house_characters = new ArrayList<Character>();
        var mayor = new Character(
                "Starosta",
                mayor_house,
                "Ahoj já jsem starosta. Dej mi moje ztracené zlaté hodinky a dám ti fragment klíče."
        );
        mayor.setInteraction(Interaction.Mayor);
        mayor.setInteractionData(new cz.jzitnik.chronos.entities.Interaction(
                "",
                "Zlaté hodinky už mám. Děkuji za nalezení.",
                mayor
        ));
        mayor_house_characters.add(mayor);
        mayor_house.setCharacters(mayor_house_characters);

        // Zarostlá zahrada
        var garden = new Room(
                "Zarostlá zahrada",
                game
        );
        // Zlaté hodinky pro starostu
        garden.getItems().add(new Item(ItemType.GOLDEN_WATCH, garden));


        rooms.add(outside);
        rooms.add(stodola);
        rooms.add(shop);
        rooms.add(library);
        rooms.add(inn);
        rooms.add(mayor_house);
        rooms.add(garden);

        game.setRooms(rooms);

        // Return a room that all players will spawn in
        return outside;
    }
}