package cz.jzitnik.game.interactions;

import cz.jzitnik.api.ApiService;
import cz.jzitnik.api.types.Character;
import cz.jzitnik.game.interactions.list.*;
import cz.jzitnik.utils.Cli;

import java.io.IOException;

public class Interactions {
    public static void runInteraction(Character character, ApiService apiService, String playerKey) throws IOException {
        var response = apiService.getInteractionData(playerKey, character.getId()).execute();
        var interactionData = response.body().getData().get();

        if (character.isInteractedWith()) {
            Cli.type(character, interactionData.getInteractedWithText());
            return;
        }

        if (!interactionData.getStartText().isBlank()) {
            Cli.type(character, interactionData.getStartText());
        }

        switch (character.getInteraction()) {
            case Farmer -> {
                RockPaperScissors rockPaperScissors = new RockPaperScissors(character, apiService, playerKey);
                rockPaperScissors.play();
            }
            case Cashier -> {
                Hangman hangman = new Hangman(character, apiService, playerKey);
                hangman.play();
            }
            case Librarian -> {
                Wordle wordle = new Wordle(character, apiService, playerKey);
                wordle.play();
            }
            case Innkeeper -> {
                TicTacToe ticTacToe = new TicTacToe(character, apiService, playerKey);
                ticTacToe.play();
            }
            case Mayor -> {
                Mayor mayor = new Mayor(character, apiService, playerKey);
                mayor.play();
            }
        }
    }
}
